{ *************************************************************************************
   Programa para hacer pruebas de conexión entre PC y Arduino a través del puerto USB
  *************************************************************************************
   El Arduino tiene conectados un LED, un pulsador y un display LCD de 16x2 caracteres.
   Al Arduino se le envían instrucciones como:

     "on" y "off" para encender y apagar el LED. Arduino devuelve por el USB el
     parámetro recibido.

     "lcd" + 16 caracteres de la primera Línea + 16 caracteres de la segunda línea
     para mostrar textos en el display LCD.

     "pregunta" + texto a preguntar. El Arduino devuelve por USB la pregunta recibida.

     Cuando se usa el pulsador en el Arduino se envía al PC el texto "PULSADOR1".

     Desde el momento que se pulsa el botón "Conectar" se crea un Thread para recibir
     todo lo que envía Arduino y mostrarlo en los campos TMemo. En un Memo se muestran
     los bytes de los datos recibidos y en otro se muestran los textos de esos datos.

   JIDS 05/03/2023
}
unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls,
  synaser;

type
  { THilo }

  THilo = class(TThread)
  private
    fStatusText: string;
    procedure ShowStatus;
  protected
    procedure Execute; override;
  public
    constructor Create(CreateSuspended: boolean);
  end;

  { TForm1 }

  TForm1 = class(TForm)
    Btn_DESCONECTAR: TButton;
    Btn_LEDOFF: TButton;
    Btn_PREGUNTAR: TButton;
    Btn_LCD: TButton;
    Btn_CONECTAR: TButton;
    Btn_LEDON: TButton;
    Btn_SALIR: TButton;
    Btn_LIMPIAR: TButton;
    Btn_INSTRUCCIONES: TButton;
    E_CONTROLSOFTWARE: TCheckBox;
    E_BITS: TEdit;
    E_BITSPARADA: TComboBox;
    E_CONTROLHARDWARE: TCheckBox;
    E_VELOCIDAD: TComboBox;
    E_PREGUNTAR: TEdit;
    E_LCD1: TEdit;
    E_LCD2: TEdit;
    E_PUERTO: TEdit;
    E_PARIDAD: TComboBox;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    L_CONEXION: TLabel;
    Memo1: TMemo;
    Memo2: TMemo;
    procedure Btn_CONECTARClick(Sender: TObject);
    procedure Btn_DESCONECTARClick(Sender: TObject);
    procedure Btn_INSTRUCCIONESClick(Sender: TObject);
    procedure Btn_LEDOFFClick(Sender: TObject);
    procedure Btn_LEDONClick(Sender: TObject);
    procedure Btn_LIMPIARClick(Sender: TObject);
    procedure Btn_SALIRClick(Sender: TObject);
    procedure Btn_PREGUNTARClick(Sender: TObject);
    procedure Btn_LCDClick(Sender: TObject);
    procedure Desconectar;
  private

  public
    function main : integer;
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

var
  ser : TBlockSerial;
  Hilo1 : THilo;
  linea, lineabytes : string;

procedure TForm1.Btn_CONECTARClick(Sender: TObject);
var
  velocidad : integer;
  puerto  : string;
  paridad : char;
  numbits, bitsparada : byte;
  controlsoft, controlhard : boolean;
begin
  puerto := trim(E_PUERTO.Text);
  velocidad := StrToInt(E_VELOCIDAD.Text);
  numbits := StrToInt(E_BITS.Text);
  paridad := E_PARIDAD.Text[1];
  if E_BITSPARADA.Text = '1'   then bitsparada := SB1;
  if E_BITSPARADA.Text = '1.5' then bitsparada := SB1andHalf;
  if E_BITSPARADA.Text = '2'   then bitsparada := SB2;
  controlsoft := E_CONTROLSOFTWARE.Checked;
  controlhard := E_CONTROLSOFTWARE.Checked;

  ser := TBlockSerial.Create;
  try
    ser.LinuxLock := False; // Es para que el archivo de bloque no quede bloqueado si el programa se cierra antes de cerrar el puerto
    ser.Connect(puerto); // COM1: COM2: (Windows) o '/dev/ttyUSB0' (Linux)
    Sleep(25);
    ser.ConvertLineEnd := true;      // Se añade un <CR> al final de cada línea
    // ser.config(115200, 8, 'N', SB1, False, False);
    ser.config(velocidad, numbits, paridad, bitsparada, controlsoft, controlhard);
    sleep(2000);   // No menos de 500ms (para 9600 baudios) 1500 para 115200bd
    L_CONEXION.Caption := 'CONECTADO';
    L_CONEXION.Color := clGreen;
    Btn_CONECTAR.Enabled := false;
    Btn_DESCONECTAR.Enabled := true;
    Btn_LEDON.Enabled := true;
    Btn_LEDOFF.Enabled := true;
    Btn_PREGUNTAR.Enabled := true;
    Btn_LCD.Enabled := true;
  except
    L_CONEXION.Caption := 'DESCONECTADO';
    L_CONEXION.Color := clGray;
    ShowMessage('[001] Error al abrir el puerto');
  end;

  Application.ProcessMessages;

  Hilo1 := THilo.Create(True); // Con True la thread no se inicia automáticamente
  if Assigned(Hilo1.FatalException) then
    raise Hilo1.FatalException;

  // En este punto se inicializan todo lo necesario antes de que las threads se ejecuten
  Hilo1.Start;
end;

procedure TForm1.Desconectar;
begin
  try
    Hilo1.Terminate;
  except
    ShowMessage('Error al terminar el hilo de ejecución que recibe datos');
  end;

  try
    ser.free;
    L_CONEXION.Caption := 'DESCONECTADO';
    L_CONEXION.Color := clGray;
    Btn_CONECTAR.Enabled := true;
    Btn_DESCONECTAR.Enabled := false;
    Btn_LEDON.Enabled := false;
    Btn_LEDOFF.Enabled := false;
    Btn_PREGUNTAR.Enabled := false;
    Btn_LCD.Enabled := false;
  except
    ShowMessage('[002] Error al cerrar el puerto');
  end;
end;

procedure TForm1.Btn_DESCONECTARClick(Sender: TObject);
begin
  Desconectar;
end;

procedure TForm1.Btn_INSTRUCCIONESClick(Sender: TObject);
begin
  ShowMessage('**********************************************************************************' + #13 +
              'Programa para hacer pruebas de conexión entre PC y Arduino a través del puerto USB' + #13 +
              '**********************************************************************************' + #13 +
              'El Arduino tiene conectados un LED, un pulsador y un display LCD de 16x2 caracteres.' + #13 +
              'Al Arduino se le envían instrucciones como:' + #13 +
              '' + #13 +
              '"on" y "off" para encender y apagar el LED. Arduino devuelve por el USB el' + #13 +
              'parámetro recibido.' + #13 +
              '' + #13 +
              '"lcd" + 16 caracteres de la primera Línea + 16 caracteres de la segunda línea' + #13 +
              'para mostrar textos en el display LCD.' + #13 +
              '' + #13 +
              '"pregunta" + texto a preguntar. El Arduino devuelve por USB la pregunta recibida.' + #13 +
              '' + #13 +
              'Cuando se usa el pulsador en el Arduino se envía al PC el texto "PULSADOR1".' + #13 +
              '' + #13 +
              'Desde el momento que se pulsa el botón "Conectar" se crea un Thread para recibir' + #13 +
              'todo lo que envía Arduino y mostrarlo en los campos TMemo. En un Memo se muestran' + #13 +
              'los bytes de los datos recibidos y en otro se muestran los textos de esos datos.' + #13 +
              '' + #13 +
              'JIDS 05/03/2023');
end;

procedure TForm1.Btn_LEDOFFClick(Sender: TObject);
begin
  try
    ser.SendString('off');
    // sleep(150);
  except
    ShowMessage('[004] Error al enviar datos al puerto');
  end;
end;

procedure TForm1.Btn_LEDONClick(Sender: TObject);
begin
  try
    ser.SendString('on');
    // sleep(150);
  except
    ShowMessage('[003] Error al enviar datos al puerto');
  end;
end;

procedure TForm1.Btn_LIMPIARClick(Sender: TObject);
begin
  Memo1.Lines.Clear;
  Memo2.Lines.Clear;
end;

procedure TForm1.Btn_SALIRClick(Sender: TObject);
begin
  Application.Terminate;  // No usar Close para finalizar el Thread
end;

procedure TForm1.Btn_PREGUNTARClick(Sender: TObject);
begin
  try
    ser.SendString('pregunta ' + E_PREGUNTAR.Text);
    // Sleep(150);
  except
    ShowMessage('[007] Error al enviar una pregunta al Arduino');
  end;

end;

procedure TForm1.Btn_LCDClick(Sender: TObject);
var
  s, linea : ansistring;
  cad1, cad2 : string;
  i : integer;
begin
  try
    cad1 := '                ';
    cad2 := '                ';
    cad1 := E_LCD1.Text + copy(cad1, 1, 16-length(E_LCD1.Text));
    cad2 := E_LCD2.Text + copy(cad2, 1, 16-length(E_LCD2.Text));
    ser.SendString('lcd' + cad1 + cad2);
    // sleep(150);
  except
    ShowMessage('[006] Error al enviar datos a la pantalla LCD');
  end;

end;

{ THilo }

procedure THilo.ShowStatus;
// Este método es llamado por Synchronize(@ShowStatus) y, por lo tanto, ejecutado por el hilo principal
// El thread principal puede acceder a los elementos de los Forms
begin
  // Form1.Caption := fStatusText;
  // Form1.Memo2.Lines.Add(fStatusText);
  if length(lineabytes) > 0 then
  begin
    Form1.Memo1.Lines.Add(lineabytes);
    Form1.Memo1.Lines.Add('');
  end;
  if length(linea) > 0 then
  begin
    Form1.Memo2.Lines.Add(linea);
    Form1.Memo2.Lines.Add('');
  end;
  Application.ProcessMessages;
  linea := '';
  lineabytes := '';
end;

procedure THilo.Execute;
var
  newStatus : string;
begin
  fStatusText := 'Iniciando el hilo que recibe datos del puerto USB ...';
  Synchronize(@Showstatus);
  fStatusText := 'El hilo de recepción de datos del puerto USB está en ejecución ...';
  while (not Terminated) and (true) do
  begin
    try
      Form1.Main;
    except
      on e:Exception do
        ShowMessage(e.Message);
    end;

    // Código del bucle del thread principal
    newStatus:='Hora: '+FormatDateTime('YYYY-MM-DD HH:NN:SS',Now);

    if NewStatus <> fStatusText then
    begin
      fStatusText := newStatus;
      Synchronize(@Showstatus);
    end;
    sleep(50); // Alternativamente aquí el thread podría esperar por algún evento
  end;
end;

constructor THilo.Create(CreateSuspended: boolean);
begin
  FreeOnTerminate := True;
  inherited Create(CreateSuspended);
end;

function TForm1.main : integer;
var
  s : ansistring;
  i : longint;
begin
  try
    if (ser.WaitingData <> 0) then
    begin
      s := '';
      s := ser.RecvPacket(5000);  // El parámetro es el TimeOut
      // sleep(100);
      linea := '';
      lineabytes := '';

      if s <> '' then
      begin
        for i := 1 to length(s) do
        begin
          lineabytes := lineabytes + IntToStr(ord(s[i])) + ' ';
          if ((ord(s[i]) > 27) and (ord(s[i]) < 255)) then
            linea := linea + s[i];
        end;
      end;
    end;
  except
    linea := '[005] Error al intentar escuchar el puerto';
  end;
end;

end.

